package com.cnbm.packing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.cnbm.admin.utils.BaseSupportUtils;
import com.cnbm.common.exception.RenException;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;

import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.packing.dto.WorkingTimeDTO;
import com.cnbm.packing.entity.WorkingTime;
import com.cnbm.packing.mapper.WorkingTimeMapper;
import com.cnbm.packing.service.WorkingTimeServiceBiz;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 班次时间段 表
 *
 * @author codeGenerator
 * @since 2023-12-19
 */
@Service
public class WorkingTimeServiceBizImpl extends CrudServiceImpl<WorkingTimeMapper, WorkingTime, WorkingTimeDTO> implements WorkingTimeServiceBiz {

    @Autowired
    private WorkingTimeMapper mapper;

    @Override
    public QueryWrapper<WorkingTime> getWrapper(Map<String, Object> params){
        String id = (String)params.get("id");

        QueryWrapper<WorkingTime> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(id), "id", id);

        return wrapper;
    }

    @Override
    public PageData<WorkingTimeDTO> page (Map<String, Object> params){
        IPage<WorkingTime> page = baseDao.selectPage(
                getPage(params, WorkingTime.CREATE_TIME, false),
                getWrapper(params)
        );
        return getPageData(page, WorkingTimeDTO.class);
    }

    @Override
    public WorkingTimeDTO get(Long id) {
        WorkingTime entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, WorkingTimeDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(WorkingTimeDTO dto) {
        //验证时间段不重合
        timeOverlap(dto);
        WorkingTime entity = ConvertUtils.sourceToTarget(dto, WorkingTime.class);
        BaseSupportUtils.setCommonField(entity);
        insert(entity);
    }

    public void timeOverlap(WorkingTimeDTO newTime){
        List<WorkingTimeDTO> dtoList = list();
        LocalTime newBeginTime = newTime.getBeginTime().toLocalTime();
        LocalTime newEndTime = newTime.getEndTime().toLocalTime();
        //不跨天
        if(newBeginTime.isBefore(newEndTime)) {
            for(WorkingTimeDTO oldTime: dtoList) {
                LocalTime beginTime = oldTime.getBeginTime().toLocalTime();
                LocalTime endTime = oldTime.getEndTime().toLocalTime();
                //不重叠算法：A.end< B.start || A.start > B.end
                if(! (newEndTime.isBefore(beginTime)|| newBeginTime.isAfter(endTime))){
                    throw new RenException("班次时间重合");
                }
            }
        }
        //跨天
        else{
            //是否存在跨天班次
            long count = dtoList.stream().filter(e->e.getBeginTime().toLocalTime().isAfter(e.getEndTime().toLocalTime())).count();
            if(count>0){
                throw new RenException("班次时间重合");
            }
            else{
                for(WorkingTimeDTO oldTime: dtoList) {
                    LocalTime beginTime = oldTime.getBeginTime().toLocalTime();
                    LocalTime endTime = oldTime.getEndTime().toLocalTime();
                    //不重叠算法：A.end< B.start || A.start > B.end
                    if(! (newEndTime.isBefore(beginTime)|| newBeginTime.isAfter(endTime))){
                        throw new RenException("班次时间重合");
                    }
                }
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(WorkingTimeDTO dto) {
        WorkingTime entity = ConvertUtils.sourceToTarget(dto, WorkingTime.class);
        BaseSupportUtils.setUpdateCommonField(entity);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<WorkingTimeDTO> list() {
        List<WorkingTimeDTO> list = mapper.list();
        return list;
    }

    @Override
    public String getOrderName(LocalDateTime time) {
        LocalTime localTime = time.toLocalTime();
        List<WorkingTimeDTO> resultList = new ArrayList<>();
        List<WorkingTimeDTO> listAll = mapper.list();
        for(WorkingTimeDTO dto : listAll){
            LocalTime beginTime = dto.getBeginTime().toLocalTime();
            LocalTime endTime = dto.getEndTime().toLocalTime();
            if(beginTime.isBefore(endTime)) {
                if (localTime.isBefore(endTime) && localTime.isAfter(beginTime)) {
                    resultList.add(dto);
                }
            }
            else if(localTime.isAfter(endTime)||localTime.isBefore(beginTime)){
                resultList.add(dto);
            }
        }
        String orderName = null;
        if(resultList.size()>0){
            orderName = resultList.get(0).getOrderName();
        }
        return orderName;
    }



}